import numpy as np
import pandas as pd
import matplotlib
import os.path

import matplotlib.pyplot as plt
import matplotlib.font_manager as fm
from io import BytesIO
import base64
from pkg_resources import resource_filename
from pandas.plotting import register_matplotlib_converters

from sweetviz import sv_html_formatters

register_matplotlib_converters()
#matplotlib.use('SVG')
#matplotlib.use('tkagg')
plt.rcParams['svg.fonttype'] = 'path'
# plt.rcParams['svg.fonttype'] = 'none'
#plt.rcParams['datapath'] = r"S:\FB_CONDA3\FBC3\lib\site-packages\matplotlib\mpl-data"
#plt.rcParams['module'] = r"module://ipykernel.pylab.backend_inline"

COLOR_TARGET_SOURCE = "#004bd1"
COLOR_TARGET_COMPARE = "#e54600"

class Graph:
    def __init__(self):
        self.index_for_css = "graph"
        # Some graphs save the underlying data
        self.data = {}
        return

    @staticmethod
    def get_encoded_base64(figure):
        as_raw_bytes = BytesIO()
        figure.savefig(as_raw_bytes, format='png', transparent=True)
        as_raw_bytes.seek(0)
        return base64.b64encode(as_raw_bytes.read())

    @staticmethod
    def set_style(style_filename_list):
        #matplotlib.style.use("classic")

        graph_font_filename = resource_filename(__name__, os.path.join("fonts", "Roboto-Medium.ttf"))
        font_dirs = [resource_filename(__name__, "fonts"), ]
        font_files = fm.findSystemFonts(fontpaths=font_dirs)
        font_list = fm.createFontList(font_files)
        fm.fontManager.ttflist.extend(font_list)
        fm.fontManager.ttflist = font_list

        styles_in_final_location = list()
        for source_name in style_filename_list:
            styles_in_final_location.append(resource_filename(__name__, os.path.join("mpl_styles",
                                                                                     source_name)))
        # fm.FontProperties(fname=graph_font_filename)
        matplotlib.style.use(styles_in_final_location)

    @staticmethod
    def format_smart(x, pos=None):
        return sv_html_formatters.fmt_smart(x)
